@@ -11,7 +11,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import base64
 import json
 
 from django.core import urlresolvers
@@ -22,6 +21,7 @@ from horizon import exceptions
 from horizon import forms
 from horizon import workflows
 
+from sahara_dashboard.api import designate as designateclient
 from sahara_dashboard.api import manila as manilaclient
 from sahara_dashboard.api import sahara as saharaclient
 from sahara_dashboard.content.data_processing.utils import helpers as helpers
@@ -31,6 +31,7 @@ from sahara_dashboard.content.data_processing.utils \
     import acl as acl_utils
 import sahara_dashboard.content.data_processing. \
     utils.workflow_helpers as whelpers
+from sahara_dashboard import utils
 
 
 class SelectPluginAction(workflows.Action,
@@ -117,11 +118,16 @@ class GeneralConfigAction(workflows.Action):
 
     def get_help_text(self):
         extra = dict()
-        plugin, hadoop_version = whelpers\
+        plugin_name, hadoop_version = whelpers\
             .get_plugin_and_hadoop_version(self.request)
 
-        extra["plugin_name"] = plugin
+        extra["plugin_name"] = plugin_name
         extra["hadoop_version"] = hadoop_version
+
+        plugin = saharaclient.plugin_get_version_details(
+            self.request, plugin_name, hadoop_version)
+        extra["deprecated"] = whelpers.is_version_of_plugin_deprecated(
+            plugin, hadoop_version)
         return super(GeneralConfigAction, self).get_help_text(extra)
 
     def clean(self):
@@ -160,12 +166,15 @@ class ConfigureNodegroupsAction(workflows.Action):
     def __init__(self, request, *args, **kwargs):
         super(ConfigureNodegroupsAction, self). \
             __init__(request, *args, **kwargs)
-
-        plugin = request.REQUEST.get("plugin_name")
-        version = request.REQUEST.get("hadoop_version")
+        # when we copy or edit a cluster template then
+        # request contains valuable info in both GET and POST methods
+        req = request.GET.copy()
+        req.update(request.POST)
+        plugin = req.get("plugin_name")
+        version = req.get("hadoop_version")
         if plugin and not version:
             version_name = plugin + "_version"
-            version = request.REQUEST.get(version_name)
+            version = req.get(version_name)
 
         if not plugin or not version:
             self.templates = saharaclient.nodegroup_template_find(request)
@@ -173,27 +182,22 @@ class ConfigureNodegroupsAction(workflows.Action):
             self.templates = saharaclient.nodegroup_template_find(
                 request, plugin_name=plugin, hadoop_version=version)
 
-        deletable = request.REQUEST.get("deletable", dict())
+        deletable = req.get("deletable", dict())
 
-        request_source = None
-        if 'forms_ids' in request.POST:
-                request_source = request.POST
-        elif 'forms_ids' in request.REQUEST:
-                request_source = request.REQUEST
-        if request_source:
+        if 'forms_ids' in req:
             self.groups = []
-            for id in json.loads(request_source['forms_ids']):
+            for id in json.loads(req['forms_ids']):
                 group_name = "group_name_" + str(id)
                 template_id = "template_id_" + str(id)
                 count = "count_" + str(id)
                 serialized = "serialized_" + str(id)
-                self.groups.append({"name": request_source[group_name],
-                                    "template_id": request_source[template_id],
-                                    "count": request_source[count],
+                self.groups.append({"name": req[group_name],
+                                    "template_id": req[template_id],
+                                    "count": req[count],
                                     "id": id,
                                     "deletable": deletable.get(
-                                        request_source[group_name], "true"),
-                                    "serialized": request_source[serialized]})
+                                        req[group_name], "true"),
+                                    "serialized": req[serialized]})
 
                 whelpers.build_node_group_fields(self,
                                                  group_name,
@@ -277,6 +281,40 @@ class SelectClusterShares(workflows.Step):
         return context
 
 
+class SelectDnsDomainsAction(workflows.Action):
+    domain_name = forms.DynamicChoiceField(
+        label=_("Domain Name"),
+        required=False
+    )
+
+    def __init__(self, request, *args, **kwargs):
+        super(SelectDnsDomainsAction, self).__init__(request, *args, **kwargs)
+
+    def _get_domain_choices(self, request):
+        domains = designateclient.get_domain_names(request)
+        choices = [(None, _('No domain is specified'))]
+        choices.extend(
+            [(domain.get('name'), domain.get('name')) for domain in domains])
+        return choices
+
+    def populate_domain_name_choices(self, request, context):
+        return self._get_domain_choices(request)
+
+    class Meta(object):
+        name = _("DNS Domain Names")
+        help_text_template = (
+            "cluster_templates/_config_domain_names_help.html")
+
+
+class SelectDnsDomains(workflows.Step):
+    action_class = SelectDnsDomainsAction
+
+    def contribute(self, data, context):
+        for k, v in data.items():
+            context["dns_" + k] = v
+        return context
+
+
 class ConfigureClusterTemplate(whelpers.ServiceParametersWorkflow,
                                whelpers.StatusFormatMixin):
     slug = "configure_cluster_template"
@@ -305,6 +343,9 @@ class ConfigureClusterTemplate(whelpers.ServiceParametersWorkflow,
         if saharaclient.base.is_service_enabled(request, 'share'):
             ConfigureClusterTemplate._register_step(self, SelectClusterShares)
 
+        if saharaclient.base.is_service_enabled(request, 'dns'):
+            ConfigureClusterTemplate._register_step(self, SelectDnsDomains)
+
         self._populate_tabs(general_parameters, service_parameters)
 
         super(ConfigureClusterTemplate, self).__init__(request,
@@ -339,7 +380,7 @@ class ConfigureClusterTemplate(whelpers.ServiceParametersWorkflow,
                 raw_ng = context.get("ng_serialized_" + str(id))
 
                 if raw_ng and raw_ng != 'null':
-                    ng = json.loads(base64.urlsafe_b64decode(str(raw_ng)))
+                    ng = json.loads(utils.deserialize(str(raw_ng)))
                 else:
                     ng = dict()
                 ng["name"] = name
@@ -355,6 +396,10 @@ class ConfigureClusterTemplate(whelpers.ServiceParametersWorkflow,
             if "ct_shares" in context:
                 ct_shares = context["ct_shares"]
 
+            domain = context.get('dns_domain_name', None)
+            if domain == 'None':
+                domain = None
+
             # TODO(nkonovalov): Fix client to support default_image_id
             saharaclient.cluster_template_create(
                 request,
@@ -368,7 +413,8 @@ class ConfigureClusterTemplate(whelpers.ServiceParametersWorkflow,
                 use_autoconfig=context['general_use_autoconfig'],
                 shares=ct_shares,
                 is_public=context['general_is_public'],
-                is_protected=context['general_is_protected']
+                is_protected=context['general_is_protected'],
+                domain_name=domain
             )
 
             hlps = helpers.Helpers(request)
