@@ -11,7 +11,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import logging
+from oslo_log import log as logging
 
 from django.core.exceptions import ValidationError
 from django.utils import safestring
@@ -25,6 +25,7 @@ from openstack_dashboard.api import network
 
 from sahara_dashboard.api import sahara as saharaclient
 
+
 LOG = logging.getLogger(__name__)
 
 
@@ -202,10 +203,13 @@ def get_security_groups(request, security_group_ids):
 def get_plugin_and_hadoop_version(request):
     plugin_name = None
     hadoop_version = None
-    if request.REQUEST.get("plugin_name"):
-        plugin_name = request.REQUEST["plugin_name"]
-        hadoop_version = request.REQUEST["hadoop_version"]
-    return (plugin_name, hadoop_version)
+    # In some cases request contains valuable info in both GET and POST methods
+    req = request.GET.copy()
+    req.update(request.POST)
+    if req.get("plugin_name"):
+        plugin_name = req["plugin_name"]
+        hadoop_version = req["hadoop_version"]
+    return plugin_name, hadoop_version
 
 
 def clean_node_group(node_group):
@@ -246,6 +250,7 @@ def populate_image_choices(self, request, context, empty_choice=False):
 class PluginAndVersionMixin(object):
     def _generate_plugin_version_fields(self, sahara):
         plugins = sahara.plugins.list()
+        plugins = filter(is_plugin_not_hidden_for_user, plugins)
         plugin_choices = [(plugin.name, plugin.title) for plugin in plugins]
 
         self.fields["plugin_name"] = forms.ChoiceField(
@@ -256,10 +261,13 @@ class PluginAndVersionMixin(object):
                        'data-slug': 'pluginname'}))
 
         for plugin in plugins:
+            version_choices = (sorted(
+                [(version, version)
+                 for version in get_enabled_versions(plugin)], reverse=True))
             field_name = plugin.name + "_version"
             choice_field = forms.ChoiceField(
                 label=_("Version"),
-                choices=[(version, version) for version in plugin.versions],
+                choices=version_choices,
                 widget=forms.Select(
                     attrs={"class": "plugin_version_choice switched "
                            + field_name + "_choice",
@@ -414,3 +422,36 @@ class MultipleShareChoiceField(forms.MultipleChoiceField):
             raise ValidationError(
                 _("The value of shares must be a list of values")
             )
+
+
+def is_plugin_not_hidden_for_user(plugin):
+    hidden_lbl = plugin.plugin_labels.get('hidden')
+    if hidden_lbl and hidden_lbl['status']:
+        return False
+    if get_enabled_versions(plugin):
+        return True
+    return False
+
+
+def get_enabled_versions(plugin):
+    lbs = plugin.version_labels
+
+    versions = []
+    for version, data in six.iteritems(lbs):
+        if data.get('enabled', {'status': True}).get('status', True):
+            versions.append(version)
+
+    if not plugin.plugin_labels.get(
+            'enabled', {'status': True}).get('status', True):
+        versions = []
+    return versions
+
+
+def is_version_of_plugin_deprecated(plugin, version):
+    lbs = plugin.version_labels
+    for iter_version, data in six.iteritems(lbs):
+        if iter_version == version:
+            if data.get('deprecated', {'status': False}).get('status', False):
+                return True
+            else:
+                return False
